unit psBarcodeWin;

interface

uses Classes, Windows, Graphics, Controls, Messages,
    psTypes,
    psCodeExceptions,
    psBarcodeComp;

type
  TpsBarcodeWinControl=class(TCustomControl, IpsBarcodeInterface)
    private
      FBarcodeComponent: TpsBarcodeComponent;

    protected

      function  GetAbout:String;

      function  BarcodeComponent:TpsBarcodeComponent;

      function  GetOptions:TpsBarcodeOptions;
      procedure SetOptions(const Value:TpsBarcodeOptions);

      function  GetBackgroundColor: TColor;
      procedure SetBackgroundColor(const Value:TColor);

      function  GetAngle: Integer;
      procedure SetAngle(const Value:Integer);

      function  GetBarcode:String;
      procedure SetBarcode(const Value:String);

      function  GetBarcodeSymbology:TpsBarcodeSymbology;
      procedure SetBarcodeSymbology(const value :TpsBarcodeSymbology);

      function  GetLinesColor:TColor;
      procedure SetLinesColor(const Value:TColor);

      function  GetQuietZone:TpsQuietZone;
      procedure SetQuietZone(const Value:TpsQuietZone);

      function  GetHorzLines:TpsHorzLines;
      procedure SetHorzLines(const Value:TpsHorzLines);

      function  GetParams:TpsParams;
      procedure SetParams(const Value:TpsParams);

      function  GetCaptionUpper:TpsBarcodeCaption;
      procedure SetCaptionUpper(const Value:TpsBarcodeCaption);

      function  GetCaptionBottom:TpsBarcodeCaption;
      procedure SetCaptionBottom(const Value:TpsBarcodeCaption);

      function  GetCaptionHuman:TpsBarcodeCaption;
      procedure SetCaptionHuman(const Value:TpsBarcodeCaption);

      function  GetErrorInfo:TpsBarcodeException;
    public
      constructor Create(AOwner:TComponent); override;
      destructor  Destroy; override;

      procedure   UpdateBarcodeWin; virtual;
      procedure   Invalidate; override;
      procedure   Paint; override;
    published
    // ----------------------------------------------------------------------
    // psBarcode properties
    // ----------------------------------------------------------------------

    property  Width;
    property  Height;

    property  CaptionHuman:TpsBarcodeCaption read GetCaptionHuman write SetCaptionHuman;
    property  CaptionBottom:TpsBarcodeCaption read GetCaptionBottom write SetCaptionBottom;
    property  CaptionUpper:TpsBarcodeCaption read GetCaptionUpper write SetCaptionUpper;
    property  Params:TpsParams read GetParams write SetParams;
    property  HorzLines:TpsHorzLines read GetHorzLines write SetHorzLines;
    property  QuietZone:TpsQuietZone read GetQuietZone write SetQuietZone;
    property  LinesColor:TColor read GetLinesColor write SetLinesColor;
    property  BarcodeSymbology:TpsBarcodeSymbology read GetBarcodeSymbology write SetBarcodeSymbology;
    property  Barcode:String read GetBarcode write SetBarcode;
    property  Options:TpsBarcodeOptions read GetOptions write SetOptions;
    property  BackgroundColor:TColor read GetBackgroundColor write SetBackgroundColor;
    property  Angle:Integer read GetAngle write SetAngle;
    property  About:String  read GetAbout;
    property  ErrorInfo:TpsBarcodeException read GetErrorInfo;

  end;

implementation

{ TpsBarcodeWinControl }

function TpsBarcodeWinControl.BarcodeComponent: TpsBarcodeComponent;
begin
  Result := FBarcodeComponent;
end;

constructor TpsBarcodeWinControl.Create(AOwner: TComponent);
begin
  inherited;
  FBarcodeComponent:=TpsBarcodeComponent.Create(self);
end;

destructor TpsBarcodeWinControl.Destroy;
begin
  FBarcodeComponent.Free;
  inherited;
end;

function TpsBarcodeWinControl.GetAbout: String;
begin
  Result := FBarcodeComponent.About;
end;

function TpsBarcodeWinControl.GetAngle: Integer;
begin
  Result := FBarcodeComponent.Angle;
end;

function TpsBarcodeWinControl.GetBackgroundColor: TColor;
begin
  Result := FBarcodeComponent.BackgroundColor;
end;

function TpsBarcodeWinControl.GetBarcode: String;
begin
  Result := FBarcodeComponent.Barcode;
end;

function TpsBarcodeWinControl.GetBarcodeSymbology: TpsBarcodeSymbology;
begin
  Result := FBarcodeComponent.BarcodeSymbology;
end;

function TpsBarcodeWinControl.GetCaptionBottom: TpsBarcodeCaption;
begin
  Result := FBarcodeComponent.CaptionBottom;
end;

function TpsBarcodeWinControl.GetCaptionHuman: TpsBarcodeCaption;
begin
  Result := FBarcodeComponent.CaptionHuman;
end;

function TpsBarcodeWinControl.GetCaptionUpper: TpsBarcodeCaption;
begin
  Result := FBarcodeComponent.CaptionUpper;
end;

function TpsBarcodeWinControl.GetErrorInfo: TpsBarcodeException;
begin
  Result := FBarcodeComponent.ErrorInfo;
end;

function TpsBarcodeWinControl.GetHorzLines: TpsHorzLines;
begin
  Result := FBarcodeComponent.HorzLines;
end;

function TpsBarcodeWinControl.GetLinesColor: TColor;
begin
  Result := FBarcodeComponent.LinesColor;
end;

function TpsBarcodeWinControl.GetOptions: TpsBarcodeOptions;
begin
  Result := FBarcodeComponent.Options;
end;

function TpsBarcodeWinControl.GetParams: TpsParams;
begin
  Result := FBarcodeComponent.Params;
end;

function TpsBarcodeWinControl.GetQuietZone: TpsQuietZone;
begin
  Result := FBarcodeComponent.QuietZone;
end;

procedure TpsBarcodeWinControl.Invalidate;
begin
  inherited;

  // PaintBarcode(Canvas , ClientRect, FBarcodeComponent);
end;


procedure TpsBarcodeWinControl.Paint;
begin
  PaintBarcode(Canvas , ClientRect, FBarcodeComponent);
end;

procedure TpsBarcodeWinControl.SetAngle(const Value: Integer);
begin
  FBarcodeComponent.Angle := Value;
  UpdateBarcodeWin
end;

procedure TpsBarcodeWinControl.SetBackgroundColor(const Value: TColor);
begin
  FBarcodeComponent.BackGroundColor := Value;
  UpdateBarcodeWin
end;

procedure TpsBarcodeWinControl.SetBarcode(const Value: String);
begin
  FBarcodeComponent.Barcode := Value;
  UpdateBarcodeWin
end;

procedure TpsBarcodeWinControl.SetBarcodeSymbology(
  const value: TpsBarcodeSymbology);
begin
  FBarcodeComponent.BarcodeSymbology := Value;
  UpdateBarcodeWin
end;

procedure TpsBarcodeWinControl.SetCaptionBottom(const Value: TpsBarcodeCaption);
begin
  FBarcodeComponent.CaptionBottom.Assign(Value);
  UpdateBarcodeWin
end;

procedure TpsBarcodeWinControl.SetCaptionHuman(const Value: TpsBarcodeCaption);
begin
  FBarcodeComponent.CaptionHuman.Assign(Value);
  UpdateBarcodeWin
end;

procedure TpsBarcodeWinControl.SetCaptionUpper(const Value: TpsBarcodeCaption);
begin
  FBarcodeComponent.CaptionUpper.Assign(Value);
  UpdateBarcodeWin
end;

procedure TpsBarcodeWinControl.SetHorzLines(const Value: TpsHorzLines);
begin
  FBarcodeComponent.HorzLines.Assign(Value);
  UpdateBarcodeWin
end;

procedure TpsBarcodeWinControl.SetLinesColor(const Value: TColor);
begin
  FBarcodeComponent.LinesColor := Value;
  UpdateBarcodeWin
end;

procedure TpsBarcodeWinControl.SetOptions(const Value: TpsBarcodeOptions);
begin
  FBarcodeComponent.Options := Value;
  UpdateBarcodeWin
end;

procedure TpsBarcodeWinControl.SetParams(const Value: TpsParams);
begin
  FBarcodeComponent.Params.Assign(Value);
  UpdateBarcodeWin
end;

procedure TpsBarcodeWinControl.SetQuietZone(const Value: TpsQuietZone);
begin
  FBarcodeComponent.QuietZone.Assign(Value);
  UpdateBarcodeWin
end;

procedure TpsBarcodeWinControl.UpdateBarcodeWin;
begin
  Invalidate;
end;

end.
